#pragma parameter frame_nonono       "FRAME:"                           0.0  0.0 1.0 1.0
#pragma parameter box_scale          "    Image Scale"                  4.0 1.0 10.0 0.02
#pragma parameter in_res_x           "    Viewport Size X"            320.0 100.0 600.0 1.0
#pragma parameter in_res_y           "    Viewport Size Y"            240.0 64.0 512.0 1.0
#pragma parameter geom_center_x      "    Center X"                     0.0 -1.0 1.0 0.001
#pragma parameter geom_center_y      "    Center Y"                     0.0 -1.0 1.0 0.002
#pragma parameter ub_rotate          "    Rotate (TATE)"                0.0  0.0 1.0 1.0

#pragma parameter geom_nonono        "GEOM'S CURVATURE:"                0.0  0.0 1.0 1.0
#pragma parameter geom_curvature     "    Curvature Toggle"             0.0 0.0 1.0 1.0
#pragma parameter geom_R             "    Curvature Radius"             2.0 0.3 10.0 0.1
#pragma parameter geom_d             "    Distance"                     1.5 0.1 3.0 0.1
#pragma parameter geom_invert_aspect "    Curvature Aspect Inversion"   0.0 0.0 1.0 1.0
#pragma parameter geom_cornersize    "    Corner Size"                  0.006 0.006 1.0 0.005
#pragma parameter geom_cornersmooth  "    Corner Smoothness"          400.0 80.0 2000.0 100.0
#pragma parameter geom_x_tilt        "    Horizontal Tilt"              0.0 -0.5 0.5 0.01
#pragma parameter geom_y_tilt        "    Vertical Tilt"                0.0 -0.5 0.5 0.01
#pragma parameter geom_inner_center_x "    Inner Center X"              0.0 -1.0 1.0 0.001
#pragma parameter geom_inner_center_y "    Inner Center Y"              0.0 -1.0 1.0 0.001
#pragma parameter geom_overscan_x    "    Horiz. Overscan %"          100.0 -125.0 125.0 0.5
#pragma parameter geom_overscan_y    "    Vert. Overscan %"           100.0 -125.0 125.0 0.5
