#pragma parameter hcrt_title                         "SONY MEGATRON COLOUR VIDEO MONITOR"                           0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_space0                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support0                      "SDR mode: Turn up your TV's brightness as high as possible"   0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support1                      "HDR mode: Set the peak luminance to that of your TV."         0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support2                      "Then adjust paper white luminance until it looks right"       0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_space1                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support3                      "Default white points for the different colour systems:"       0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support4                      "709: D65, PAL: D65, NTSC-U: D65, NTSC-J: D93"         		0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_space2                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support5                      "Rec. 709 gamma cutoff is 0.018 x 1000 = 18"       			0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support6                      "sRGB gamma cutoff is 0.0031308 x 1000 = ~31"       			0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_support7                      "Rec. 601/709 inverse gamma cutoff is 0.081 x 1000 = 81"       0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_space3                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_user_settings                 "YOUR DISPLAY'S SETTINGS:"                                     0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_hdr                           "    SDR | HDR"                                                1.0      0.0   1.0      1.0
#pragma parameter hcrt_space4                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_max_nits                      "    HDR: Display's Peak Luminance"                            1000.0   0.0   10000.0  10.0
#pragma parameter hcrt_paper_white_nits              "    HDR: Display's Paper White Luminance"                     200.0    0.0   10000.0  10.0
#pragma parameter hcrt_expand_gamut                  "    HDR: Original/Vivid"                                      0.0      0.0   1.0      1.0
#pragma parameter hcrt_space5                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_colour_space                  "    SDR: Display's Colour Space: r709 | sRGB | DCI-P3"        1.0      0.0   2.0      1.0
#pragma parameter hcrt_r709_gamma_out                "    SDR: r709 Gamma"                                          2.22     1.0   5.0      0.01
#pragma parameter hcrt_srgb_gamma_out                "    SDR: sRGB Gamma"                                          2.4      1.0   5.0      0.01
#pragma parameter hcrt_p3_gamma_out                  "    SDR: DCI-P3 Gamma"                                        2.6      1.0   5.0      0.01
#pragma parameter hcrt_r709_gamma_cutoff             "    SDR: r709 Gamma Cutoff (x1000)"                           1.0      0.0   100.0    1.00
#pragma parameter hcrt_srgb_gamma_cutoff             "    SDR: sRGB Gamma Cutoff (x1000)"                           1.0      0.0   100.0    1.00
#pragma parameter hcrt_space6                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_lcd_resolution                "    Display's Resolution: 1080p | 4K | 8K"                    1.0      0.0   2.0      1.0
#pragma parameter hcrt_lcd_subpixel                  "    Display's Subpixel Layout: RGB | RWBG (OLED) | BGR"       0.0      0.0   2.0      1.0
#pragma parameter hcrt_colour_accurate               "    Mask Accurate/Colour Accurate"                            1.0      0.0   1.0      1.0
#pragma parameter hcrt_space7                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_developer_settings            "CRT SETTINGS:"                                                0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_crt_screen_type               "    Screen Type: APERTURE GRILLE | SHADOW MASK | SLOT MASK"   0.0      0.0   2.0      1.0
#pragma parameter hcrt_crt_resolution                "    Resolution: 300TVL | 600TVL | 800TVL | 1000TVL"           1.0      0.0   3.0      1.0
#pragma parameter hcrt_colour_system                 "    Colour System: r709 | PAL | NTSC-U | NTSC-J"              2.0      0.0   3.0      1.0
#pragma parameter hcrt_phosphor_set                  "    Phosphors: NONE | P22-J | P22-80 | P22-90 | RPTV | 1953"  0.0      0.0   5.0      1.0
#pragma parameter hcrt_white_temperature_D65         "    D65 White Temperature (Kelvin)"                    		6504.0   0.0   18500.0  100.0
#pragma parameter hcrt_white_temperature_D93         "    D93 White Temperature (Kelvin)"                    		9305.0   0.0   18500.0  100.0
#pragma parameter hcrt_brightness                    "    Brightness"                                               0.0     -1.0   1.0      0.01
#pragma parameter hcrt_contrast                      "    Contrast"                                                 0.0     -1.0   1.0      0.01
#pragma parameter hcrt_saturation                    "    Saturation"                                               0.0     -1.0   1.0      0.01
#pragma parameter hcrt_gamma_in                      "    Gamma"                                                    2.22     1.0   5.0      0.01
#pragma parameter hcrt_gamma_cutoff                  "    Inverse Gamma Cutoff (x1000)"                             1.0      0.0   100.0    1.0
#pragma parameter hcrt_pin_phase                     "    Pin Phase"                                                0.00    -0.2   0.2      0.01
#pragma parameter hcrt_pin_amp                       "    Pin Amp"                                                  0.00    -0.2   0.2      0.01
#pragma parameter hcrt_space8                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_developer_settings0           "    VERTICAL SETTINGS:"                                       0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_v_size                        "        Vertical Size"                                        1.00     0.8   1.2      0.01
#pragma parameter hcrt_v_cent                        "        Vertical Center"                                      0.00  -200.0 200.0      1.0
#pragma parameter hcrt_red_vertical_convergence      "        Red Vertical Deconvergence"                           0.00     -10.0 10.0     0.01
#pragma parameter hcrt_green_vertical_convergence    "        Green Vertical Deconvergence"                         0.00     -10.0 10.0     0.01
#pragma parameter hcrt_blue_vertical_convergence     "        Blue Vertical Deconvergence"                          0.00     -10.0 10.0     0.01
#pragma parameter hcrt_red_scanline_min              "        Red Scanline Min"                                     0.50     0.0   2.0      0.01 
#pragma parameter hcrt_red_scanline_max              "        Red Scanline Max"                                     1.00     0.0   2.0      0.01 
#pragma parameter hcrt_red_scanline_attack           "        Red Scanline Attack"                                  0.20     0.0   1.0      0.01
#pragma parameter hcrt_green_scanline_min            "        Green Scanline Min"                                   0.50     0.0   2.0      0.01 
#pragma parameter hcrt_green_scanline_max            "        Green Scanline Max"                                   1.00     0.0   2.0      0.01 
#pragma parameter hcrt_green_scanline_attack         "        Green Scanline Attack"                                0.20     0.0   1.0      0.01
#pragma parameter hcrt_blue_scanline_min             "        Blue Scanline Min"                                    0.50     0.0   2.0      0.01 
#pragma parameter hcrt_blue_scanline_max             "        Blue Scanline Max"                                    1.00     0.0   2.0      0.01 
#pragma parameter hcrt_blue_scanline_attack          "        Blue Scanline Attack"                                 0.20     0.0   1.0      0.01
#pragma parameter hcrt_space9                        " "                                                            0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_developer_settings1           "    HORIZONTAL SETTINGS:"                                     0.0      0.0   0.0001   0.0001
#pragma parameter hcrt_h_size                        "        Horizontal Size"                                      1.00     0.8   1.2      0.01
#pragma parameter hcrt_h_cent                        "        Horizontal Center"                                    0.00  -200.0 200.0      1.0
#pragma parameter hcrt_red_horizontal_convergence    "        Red Horizontal Deconvergence"                         0.00     -10.0 10.0     0.01
#pragma parameter hcrt_green_horizontal_convergence  "        Green Horizontal Deconvergence"                       0.00     -10.0 10.0     0.01
#pragma parameter hcrt_blue_horizontal_convergence   "        Blue Horizontal Deconvergence"                        0.00     -10.0 10.0     0.01
#pragma parameter hcrt_red_beam_sharpness            "        Red Beam Sharpness"                                   1.75     0.0   5.0      0.05
#pragma parameter hcrt_red_beam_attack               "        Red Beam Attack"                                      0.50     0.0   2.0      0.01
#pragma parameter hcrt_green_beam_sharpness          "        Green Beam Sharpness"                                 1.75     0.0   5.0      0.05
#pragma parameter hcrt_green_beam_attack             "        Green Beam Attack"                                    0.50     0.0   2.0      0.01
#pragma parameter hcrt_blue_beam_sharpness           "        Blue Beam Sharpness"                                  1.75     0.0   5.0      0.05
#pragma parameter hcrt_blue_beam_attack              "        Blue Beam Attack"                                     0.50     0.0   2.0      0.01

