#ifndef HELPER_MACROS
#define HELPER_MACROS

// All of the parameter macros in one place
// replace 'global' with whichever struct you use for parameters

#define ia_FLIP_HORZ global.ia_FLIP_HORZ
#define ia_FLIP_VERT global.ia_FLIP_VERT
#define ia_overscan_percent_x global.ia_overscan_percent_x
#define ia_overscan_percent_y global.ia_overscan_percent_y
#define ia_ZOOM global.ia_ZOOM
#define ia_XPOS global.ia_XPOS
#define ia_YPOS global.ia_YPOS
#define ia_GRAIN_STR global.ia_GRAIN_STR
#define ia_SHARPEN global.ia_SHARPEN
#define ia_saturation global.ia_saturation
#define ia_luminance global.ia_luminance
#define ia_contrast global.ia_contrast
#define ia_bright_boost global.ia_bright_boost
#define ia_black_level global.ia_black_level
#define ia_target_gamma_r global.ia_target_gamma_r
#define ia_target_gamma_g global.ia_target_gamma_g
#define ia_target_gamma_b global.ia_target_gamma_b
#define ia_monitor_gamma global.ia_monitor_gamma
#define ia_R global.ia_R
#define ia_G global.ia_G
#define ia_B global.ia_B
#define ia_TOPMASK global.ia_TOPMASK
#define ia_BOTMASK global.ia_BOTMASK
#define ia_LMASK global.ia_LMASK
#define ia_RMASK global.ia_RMASK
#define ia_rr global.ia_rr
#define ia_gg global.ia_gg
#define ia_bb global.ia_bb
#define ia_rg global.ia_rg
#define ia_rb global.ia_rb
#define ia_gr global.ia_gr
#define ia_gb global.ia_gb
#define ia_br global.ia_br
#define ia_bg global.ia_bg
#define temperature global.temperature
#define luma_preserve global.luma_preserve
#define wp_red global.wp_red
#define wp_green global.wp_green
#define wp_blue global.wp_blue
#define integer_scale global.integer_scale
#define overscale global.overscale
#define ar_num global.ar_num
#define ar_den global.ar_den
#define g_hpfix global.g_hpfix
#define g_cntrst global.g_cntrst
#define g_mid global.g_mid
#define mask_strength global.mask_strength
#define mask_picker global.mask_picker
#define LUT_Size1 global.LUT_Size1
#define LUT_Size2 global.LUT_Size2
#define g_vignette global.g_vignette
#define g_vstr global.g_vstr
#define g_vpower global.g_vpower
#define gamma_in_lvl global.gamma_in_lvl
#define gamma_out_lvl global.gamma_out_lvl
#define warpX global.warpX
#define warpY global.warpY
#define cornersize global.cornersize
#define cornersmooth global.cornersmooth
#define Radius global.Radius
#define Distance global.Distance
#define x_tilt global.x_tilt
#define y_tilt global.y_tilt

#endif
