// clang-format off
#pragma parameter MIN_EXP_INTENSITY "Bloom intensity" 0.6 0.0 2.0 0.05
#pragma parameter BLOOM_THRESHOLD "Bloom threshold" 3.5 1.0 10.0 0.1
#pragma parameter ADAPT_SPEED "Eye adaptation speed" 0.1 0.0 1.0 0.01
#pragma parameter EYE_ADAPTION "Eye adaptation strength: Reduce bloom on bright screens" 0.8 0.0 1.0 0.05
#pragma parameter MIN_EXP "Minimum exposure value (where bloom is strongest)" 0.1 0.0 0.95 0.01
#pragma parameter MAX_EXP "Maximum exposure value (where bloom is weakest)" 0.55 0.05 1.0 0.01
// clang-format on