// See common.inc for copyright and other information.

// clang-format off
#pragma parameter GAUSS_4TAP_SETTINGS "=== Gauss-4tap v1.0 settings ===" 0.0 0.0 1.0 1.0
#pragma parameter SIGMA "Gaussian filtering sigma" 1.0 0.0 2.0 0.05
// clang-format on
