#ifndef PARAM_FLOATS
#define PARAM_FLOATS

// The parameters that go into your struct
// Collected here for tidiness
   float ia_FLIP_HORZ;
   float ia_FLIP_VERT;
	float ia_overscan_percent_x;
	float ia_overscan_percent_y;
	float ia_ZOOM;
	float ia_XPOS;
	float ia_YPOS;
	float ia_GRAIN_STR;
	float ia_SHARPEN;
	float ia_saturation;
	float ia_luminance;
	float ia_contrast;
	float ia_bright_boost;
	float ia_black_level;
	float ia_target_gamma_r;
	float ia_target_gamma_g;
	float ia_target_gamma_b;
	float ia_monitor_gamma;
	float ia_TOPMASK;
	float ia_BOTMASK;
	float ia_LMASK;
	float ia_RMASK;
	float ia_rr;
	float ia_gg;
	float ia_bb;
	float ia_rg;
	float ia_rb;
	float ia_gr;
	float ia_gb;
	float ia_br;
	float ia_bg;
   float temperature;
   float luma_preserve;
   float wp_red;
   float wp_green;
   float wp_blue;
   float ar_num;
   float ar_den;
   float integer_scale;
   float overscale;
   float g_hpfix;
   float g_cntrst;
   float g_mid;
   float mask_strength;
   float mask_picker;
   float LUT_Size1;
   float LUT_Size2;
   float g_vignette;
   float g_vstr;
   float g_vpower;
   float gamma_in_lvl;
   float gamma_out_lvl;
   float warpX;
   float warpY;
   float cornersize;
   float cornersmooth;
   float Radius;
   float Distance;
   float x_tilt;
   float y_tilt;
   
#endif
