// Yeah, it's redundant, but I won't create inner 'includes' over and over, because Retroarch
// crawls to unroll these f***rs.

#pragma parameter frame_nonono       "FRAME:"                           0.0  0.0 1.0 1.0
#pragma parameter box_scale          "    Image Scale"                  4.0 1.0 10.0 0.02
#pragma parameter in_res_x           "    Viewport Size X"            320.0 100.0 600.0 1.0
#pragma parameter in_res_y           "    Viewport Size Y"            240.0 64.0 512.0 1.0
#pragma parameter geom_center_x      "    Center X"                     0.0 -1.0 1.0 0.001
#pragma parameter geom_center_y      "    Center Y"                     0.0 -1.0 1.0 0.002
#pragma parameter ub_rotate          "    Rotate (TATE)"                0.0  0.0 1.0 1.0

