    uint Rotation;

    float all_zoom;

    float fr_zoom;
    float fr_aspect_x;
    float fr_aspect_y;
    float fr_center_x;
    float fr_center_y;
    float fr_i_scaling;
    float fr_i_scaling_fac;

    float h_shape;
    float h_radius;
    float h_cornersize;
    float h_cornersmooth;
    float h_overscan_x;
    float h_overscan_y;
    float h_x_tilt;
    float h_y_tilt;
    float h_angle_x;
    float h_angle_y;
    float h_curvature;

#ifdef USE_BEZEL_REFLECTIONS_COMMON

    float bz_shine_enable;
    float bz_shine;
    float bz_shine_size;
    float bz_ambient_enable;
    float bz_ambient;
    float bz_ambient_size;
    float bz_blur_iter;
    float bz_ref_str;
    float bz_lights;
    float bz_outer_transparent;
    float bz_transparent;
    float bz_inner_bezel_x;
    float bz_inner_bezel_y;
    float bz_middle_bezel_x;
    float bz_middle_bezel_y;
    float bz_outer_bezel_x;
    float bz_outer_bezel_y;
    float bz_outer_curve;
    float bz_curve_radius;
    float bz_inner_curve;
    float bz_in_curve_radius;
    float bz_ref_dist;
    float bz_radius;
    float bz_center_x;
    float bz_center_y;
    float bz_red;
    float bz_green;
    float bz_blue;
    float bz_ang;
    float bz_pos;

    float border_scale;
    float border_center_x;
    float border_center_y;
    float border_mirror_y;
    float border_allow_rot;
    float ub_border_top;
    float border_alpha;
    float black_baoc;
    float border_weight;
    float border_dimming;
    float layer2_dimming;
#endif
